// --------------------------------------------------------------------
// Copyright (c) 2005 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	DE2 TOP LEVEL
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            :| Mod. Date :| Changes Made:
//   V1.0 :| Johnny Chen       :| 05/08/19  :|      Initial Revision
//   V1.1 :| Johnny Chen       :| 05/11/16  :|      Add to FLASH Address FL_ADDR[21:20]
//   V1.2 :| Johnny Chen       :| 05/11/16  :|		Fixed ISP1362 INT/DREQ Pin Direction.   
// --------------------------------------------------------------------

module DDR_TOP
	(
		////////////////////	Clock Input	 	////////////////////	 
		CLOCK_27,						//	27 MHz
		CLOCK_50,						//	50 MHz
		////////////////////	Push Button		////////////////////
		KEY,							//	Pushbutton[3:0]
		////////////////////	7-SEG Dispaly	////////////////////
		HEX0,							//	Seven Segment Digit 0
		HEX1,							//	Seven Segment Digit 1
		HEX2,							//	Seven Segment Digit 2
		HEX3,							//	Seven Segment Digit 3
		ddr_rst, //****REMOVE WHEN COMPILING FOR FPGA****
		GPIO_0,							//	GPIO Connection 0
		GPIO_1							//	GPIO Connection 1
	);

////////////////////////	Clock Input	 	////////////////////////
input			CLOCK_27;				//	27 MHz
input			CLOCK_50;				//	50 MHz

input	[3:0]	KEY;					//	Pushbutton[3:0]

input ddr_rst;
////////////////////////	7-SEG Dispaly	////////////////////////
output	[6:0]	HEX0;					//	Seven Segment Digit 0
output	[6:0]	HEX1;					//	Seven Segment Digit 1
output	[6:0]	HEX2;					//	Seven Segment Digit 2
output	[6:0]	HEX3;					//	Seven Segment Digit 3


inout	[35:0]	GPIO_0;					//	GPIO Connection 0
inout	[35:0]	GPIO_1;					//	GPIO Connection 1

//	Turn on all display
assign	HEX0		=	7'h00;
assign	HEX1		=	7'h00;
assign	HEX2		=	7'h00;
assign	HEX3		=	7'h00;


//	All inout port turn to tri-state

//Linking wires from expansion headers to FPGA in/out

wire  [12:0] DDR_addr_w;
assign { 		    GPIO_1[16], //addr 12
					GPIO_1[17], //11
					GPIO_1[7], //10
					GPIO_1[18], //9
					GPIO_1[19], //8
					GPIO_1[20], //7 
					GPIO_1[21], //6
					GPIO_1[22], //5
					GPIO_1[23], //4
					GPIO_1[2], //3
					GPIO_1[4], //2
					GPIO_1[5], //1
					GPIO_1[6]  //addr 0
				} = DDR_addr_w; 

//assign GPIO_0[1] = 1b'1;
//assign GPIO_0[0] = 1b'0;

wire [1:0] bank_w;
assign {GPIO_1[8],GPIO_1[9]} = bank_w ; //ba1-0

reg RAS_w, CAS_w, WE_w;

wire CS_w, LDM_w,LDQS_w;


assign CS_w = 0;

assign 		{ GPIO_0[3], //cs
			GPIO_0[4], //ras
			GPIO_0[5], //cas
			GPIO_0[6], //we
			GPIO_0[7], //LDM
			GPIO_0[8] //LDQS
							} = {CS_w, RAS_w,CAS_w,WE_w,LDM_w,LDQS_w};

wire [15:0] data_io_w;
assign {	GPIO_0[21], //d15
					GPIO_0[20], //14
					GPIO_0[19], //13
					GPIO_0[18], //12
					GPIO_0[17], //11
					GPIO_0[22], //10
					GPIO_0[23], //9
					GPIO_0[24], //8
					GPIO_0[9], //7
					GPIO_0[10], //6
					GPIO_0[11], //5
					GPIO_0[16], //4
					GPIO_0[15], //3
					GPIO_0[14], //2
					GPIO_0[13], //1
					GPIO_0[12]  //0
				} = data_io_w; //bidirectional data io
					
wire UDM_w, UDQS_w, CKn_w, CK_w, CKE_w;
wire CK_w_90, CK_w_270;
assign 	GPIO_0[29:25] 
	= {CKE_w, CK_w, CKn_w, UDM_w,UDQS_w}; 
			
wire [31:0] data_out;  // to the ddr
wire [31:0] data_in;   // from the ddr
//wire ddr_clk, reset;
//assign reset = ~KEY[0]; //reset = button 0

wire inclock = CK_w; 
wire outclock = CK_w; 

assign CKn_w = ~CK_w;
//assign CK_w = CLOCK_50;
//assign GPIO_0[0] = CLOCK_50;


//assign data_in[31:16] = 16'hffff;
//assign data_in[15:0] = 16'h0000;


    // ren is 3 clocks early, to allow for output pipeline precharde
    // we delay ren for 3 clocks for the dqs testing
    reg ren, ren0, ren1;
	wire reno;
    always @(posedge CK_w or posedge ~pll_clock_locked)
    	if (~pll_clock_locked) {ren, ren0, ren1} = 0;
    	else         {ren, ren0, ren1} = {ren0, ren1, reno};

    reg ddrio_oe;
     always @(posedge CK_w or posedge ~pll_clock_locked)
    	if (~pll_clock_locked) ddrio_oe = 0;
    	else                   ddrio_oe = ren;
    	
    	
//     reg dqs_cmd;
//     //wire dqs;
//     //assign dqs = dts == 1 ? 1'bZ : dqs_cmd; 
// 
// 	assign LDQS_w = dts == 1 ? 1'bZ : dqs_cmd; 
// 	assign UDQS_w = dts == 1 ? 1'bZ : dqs_cmd; 
    
    
    
//     always @(posedge ren) begin
//     	@(posedge CK_w);
//     		ddr_ext_dat_o = 8'haa;
//     		dqs_cmd = 1;
//     	@(negedge CK_w);
//      		ddr_ext_dat_o = 8'h55;
//     		dqs_cmd = 0;
// 	end




//io to/from DDR memory

// bit 17 is used as DQS
wire dqs_in_h, dqs_in_h;
			
`ifdef SIMULATION //required for simulation to run properly
altddio_bidir 
#(.width(17),
  .intended_device_family("CYCLONE II")
 ) ddrio (
`else
altddio io (
`endif
	    .datain_h({1'b0, 16'h0000}),   // required port, input data to be output of padio port at the
	                // rising edge of outclock
	    .datain_l({1'b1, 16'h1234}),   // required port, input data to be output of padio port at the
	                // falling edge of outclock
	    .inclock(inclock),    // required port, input reference clock to sample data by
		.outclock(outclock),   // required port, input reference clock to register data output
	    .inclocken(1'b1),  // inclock enable
	    .outclocken(1'b1), // outclock enable

		//.aclr(reset),       // asynchronous clear
	    .oe(ddrio_oe),         // output enable for padio port
	    .dataout_h({dqs_in_h, data_out[31:16]}),  // data sampled from the padio port at the rising edge of inclock
	    .dataout_l({dqs_in_l, data_out[15:0]}),  // data sampled from the padio port at the falling edge of
	    .padio({LDQS_w, data_io_w})     // bidirectional DDR port
	); //bidirectional data to and from DDR ram

wire pll_clock_locked; //output from pll

//DDR clk pll
`ifdef SIMULATION
assign CK_w = CLOCK_50;
assign pll_clock_locked = !ddr_rst;
`else
write_clk_pll pll1 (
	.inclk0(CLOCK_50),
	.pllena(1'b1), //always enable 
	.c0(CK_w),
	.c1(CK_w_90),
	.c2(CK_w_270),
	.areset(ddr_rst),
	.locked(pll_clock_locked) );
`endif

    wire [1:0] ddr_cmd_rw;
    wire  ddr_cmd_init;
    
    wire [10:0] ddr_cmd_addr; // A11, 12 is always set to 0 in ddrctrl
    wire [2:0]  ddr_cmd_rcw;

    wire [2:0]  ddr_len;
    wire [12:0] ddr_row;
    wire [1:0]  ddr_ba, ba_init, ba_rw;
    


    wire [12:0] ddr_addr;

    wire [2:0] ddr_rcw;
    wire ddr_dts;

    /* UNUSED
    reg dts, dts0, dts1;
    always @(posedge CK_w or posedge ~pll_clock_locked)
    	if (~pll_clock_locked) {dts, dts0, dts1} = 0;
    	else         {dts, dts0, dts1} = {dts0, dts1, ddr_dts};*/
        
    



    // -----  IO PADS & REGISTERS ------
    
    // pad regs
    reg [7:0] ddr_ext_dat_o;


	assign ddr_ba = init_active ? ba_init : ba_rw; 

    reg [12:0]  ddr_ext_addr;
    reg [1:0]   ddr_ext_ba;
    always @(posedge CK_w)
      {ddr_ext_addr, ddr_ext_ba} <= {ddr_addr, ddr_ba};

	assign DDR_addr_w = ddr_ext_addr;
	assign bank_w = ddr_ext_ba;
    

    always @(posedge CK_w)
    	{RAS_w,CAS_w,WE_w} <= ddr_rcw; // iob register
     
    assign {UDM_w, LDM_w} = {1'b0, 1'b0}; //data mask always on *active low*

 
 	wire init_active, ddr_ctrl_done;
 
	 ddr_init init
	(
		.clk(CK_w),
		.rst(~pll_clock_locked),
		.ddr_ctrl_done(ddr_ctrl_done),
		
		.addr_o(ddr_cmd_addr), // addr[12:11] are added in the ddrctrl
		.ba_o(ba_init),
		.rcw_o(ddr_cmd_rcw),    // ras, cas, we
		.cmd_o(ddr_cmd_init),         // only one bit is required
		.init_active_o(init_active),
		.cke_o (CKE_w)
	
	);

	ddr_read_write rw
	(
		.ddr_cmd_o(ddr_cmd_rw),
		.ba_o(ba_rw),
		.burst_len_o(ddr_len),  // (number of words in burst) / 2 
		.row_o(ddr_row),
		.column_o(),
		.test_done(GPIO_0[0]),
	
		.clk(CK_w),
		.rst(~pll_clock_locked),
		.init_active (init_active),
		.ddr_ctrl_done(ddr_ctrl_done)
	);
 
    
    ddram_ctrl ddramctrl
      (
       // arbiter interface
       .arb_cmd_i   (ddr_cmd_rw),  //used to send read/write commands
       .cpu_cmd_i   (ddr_cmd_init), //used to send extended commands
      // external command
       .cmd_addr_i (ddr_cmd_addr),  // address for CMD_EXT
       .cmd_rcw_i  (ddr_cmd_rcw),   // ras, cas, we CMD_EXT


       .row_i   (ddr_row),
       .len_i   (ddr_len),
       .done_o  (ddr_ctrl_done),
        

       .ren_o   (reno), //used for DQS generation
       .wen_o   (), //not used
        
       // ddram interface
       .dts_o  (ddr_dts),  // data buffers tristate - NOT USED in this project but might be useful for someone else
       .addr_o (ddr_addr),
       .rcw_o  (ddr_rcw), // ras, cas, we

       // system   
       .CLK (CK_w),
       .RST (~pll_clock_locked)
       
       );




endmodule