


`include "defs.v"



module ddr_init
(
	input wire clk,
	input wire rst,
	input wire ddr_ctrl_done,
	
	output wire [10:0] addr_o, // addr[12:11] are added in the ddrctrl
	output wire [1:0] ba_o,
	output wire [2:0] rcw_o,  // ras, cas, we
	output wire cmd_o,         // only one bit is required
	output wire init_active_o,
	output wire cke_o

);

	reg ddr_cmd, init_active;
	reg [1:0] ddr_ba;
	reg [2:0] ddr_cmd_rcw;
	reg [1:0] Cke_cmd;
	reg [10:0] ddr_cmd_addr;
	
	
	
	assign cmd_o = ddr_cmd;
	assign init_active_o = init_active;
	assign ba_o = ddr_ba;
	assign rcw_o = ddr_cmd_rcw;
	assign addr_o = ddr_cmd_addr;
	assign cke_o  = Cke_cmd;

    parameter CMD_IDLE = 1'b0,
              CMD_EXT  = 1'b1;

	//counters for wait states
	reg [8:0] counter1, counter3;
	reg [3:0] counter2;

	reg [5:0] state, prev_state;
	parameter st_start = 5'd0, st_idle1 = 5'd1, st_pre1 = 5'd2, st_count9 = 5'd3,
			  st_setemr = 5'd4, st_setmr = 5'd5, st_pre2 = 5'd6, st_auto1 = 5'd7,
			  st_auto2 = 5'd8, st_auto3 = 5'd9, st_count200 = 5'd10, st_wait = 5'd11,
			  st_done = 5'd12;

always @(posedge clk)
begin
	if(rst)
	begin
		state <= st_start;
		init_active <= 1'b1;
		ddr_ba <= 2'b0;
		{ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} <= 15'b0;
		Cke_cmd <= 2'b0;
		counter1 <= 9'd0;
		counter2 <= 4'd0; 
	end
	else begin
		case (state)
			st_start: //init initialized - first thing is wait 500 cycles
			begin			
				if(counter1 == 500)
				begin
					state <= st_idle1;
					counter1 <= 9'd0;
				end else 
				begin
					counter1 <= counter1 + 9'b1; 
		        end				
			end //end st_start
			
		    st_idle1: //bring CKE high and execute idle command 
			begin
				Cke_cmd <= 2'b01;
				ddr_cmd <= CMD_IDLE;
				state <= st_pre1; 
			end //end st_idle 1
			
			st_count9:
			begin
				if(counter2 == 4'd9) //done counting 9 go to next state
				begin
					counter2 <= 4'b0; 
					
					case(prev_state)						
						st_pre1: state <= st_setemr;
						st_setemr: state <= st_setmr;
						st_setmr: state <= st_pre2; 
						st_pre2: state <= st_auto1;
						st_auto1: state <= st_auto2;
						st_auto2: state <= st_auto3;
					endcase
				end
				else begin
					ddr_cmd <= CMD_IDLE;
					counter2 <= counter2 + 4'b1; 
				end
			
			end //end st_count9
			
			st_pre1: //pre charge all banks
			begin
				{ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} <= {CMD_EXT,  3'b010, 11'b100_0000_0000};
				state <= st_count9; 
				counter2 <= 4'b0; 
				prev_state <= state; 
			end  //end st_pre1
			
			st_setemr:
			begin
				ddr_ba <= 2'b01; //select extended mr, set it to all zeros
				{ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b000, 11'b000_0000_0000};
				state <= st_count9;
				prev_state <= state; 
			end //end setemr
			
			st_setmr: //set mode register, defines normal operating parameters 
			begin
				 ddr_ba = 2'b00;
				// addr[N:7] - mode: ..00000 - normal op, ....10 normal op, reset  dll
				// addr[6:4] - cas latency: 010 - 2, 011 - 3 (for -5B only), 110 - 2.5
				// addr[3]   - burst type:  0 - sequential, 1 - interleave
				// addr[2:0] - burst len: 001 - 2, 010 - 4, 011 - 8
				{ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b000, 11'b0010_010_0_001};
				state <= st_count9;
				prev_state <= state; 
			end //end set mr
			
			st_pre2:
			begin
				 {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} <= {CMD_EXT,  3'b010, 11'b100_0000_0000};
				  state <= st_count9;
				  prev_state <= state; 	
			end //end pre2
			
			st_auto1: //auto referesh all banks
			begin
				 {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} <= {CMD_EXT,  3'b001, 11'b000_0000_0000};
				  state <= st_count9;
				  prev_state <= state; 
			end //end auto1
			
			st_auto2: //auto referesh all banks
			begin
				 {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} <= {CMD_EXT,  3'b001, 11'b000_0000_0000};
				  state <= st_count9;
				  prev_state <= state; 
			end //end auto2
			
			st_auto3: //auto referesh all banks
			begin
				 {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} <= {CMD_EXT,  3'b001, 11'b000_0000_0000};
				  state <= st_count200;
				  counter3 <= 9'd0;
			end //end auto3
			
			st_count200: //wait 200 cycles
			begin			
				if(counter3 == 9'd200)
				begin
					state <= st_wait;
				end else
				begin
					ddr_cmd <= CMD_IDLE;
					counter3 <= counter3 + 9'd1;
				end				
			end //end count 200
			
			st_wait:
			begin
				if(ddr_ctrl_done)
				begin
					init_active <= 1'b0; 
					state <= st_done;
				end
			end
			
			
			
		endcase //end case
		
		
	end //end if
		
	/*
		initial begin

			init_active = 1;
	        ddr_ba = 0;
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = 0;
			Cke_cmd = 0;

			wait (!rst);
			
	        #1;

	       // #(`DDR_CLK_PERIOD * 500); // wait 10 us ! after ddr_clk is functional
	        //bring CKE high and execute idle command for 1 cycle
	
	        Cke_cmd = 1;
	        ddr_cmd = CMD_IDLE; 
	        #(`DDR_CLK_PERIOD * 1);
	        // precharge all banks
	        $display ("precharge all banks at %t", $time);
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b010, 11'b100_0000_0000};
	        #(`DDR_CLK_PERIOD * 1) ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);
	        
	        //set extended mode register
	        ddr_ba = 2'b01; //select extended mr
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b000, 11'b000_0000_0000};
	         #(`DDR_CLK_PERIOD * 1)
	          ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);
	        
	        //set mode register
	        ddr_ba = 2'b00;
	        // addr[N:7] - mode: ..00000 - normal op, ....10 normal op, reset  dll
	        // addr[6:4] - cas latency: 010 - 2, 011 - 3 (for -5B only), 110 - 2.5
	        // addr[3]   - burst type:  0 - sequential, 1 - interleave
	        // addr[2"0] - burst len: 001 - 2, 010 - 4, 011 - 8
	         {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b000, 11'b0010_010_0_001};
	        #(`DDR_CLK_PERIOD * 1) ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);
	        // precharge all banks
	
			// $dumpon;
	
	        $display ("precharge all banks at %t", $time);
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b010, 11'b100_0000_0000};
	        #(`DDR_CLK_PERIOD * 1) ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);
	        // autorefresh        
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b001, 11'b000_0000_0000};
	        #(`DDR_CLK_PERIOD * 1) ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);
	        // autorefresh
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b001, 11'b000_0000_0000};
	        #(`DDR_CLK_PERIOD * 1) ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);  
	        // autorefresh
	        {ddr_cmd, ddr_cmd_rcw, ddr_cmd_addr} = {CMD_EXT,  3'b001, 11'b000_0000_0000};
	        #(`DDR_CLK_PERIOD * 1) ddr_cmd = CMD_IDLE;// REMOVE CMD !!!
	        #(`DDR_CLK_PERIOD * 9);
	             
	        //$dumpoff;
			
			#(`DDR_CLK_PERIOD * 200);
	        
	        //$dumpon;
	        wait (ddr_ctrl_done);
		
			init_active = 0;
			$finish;
		end*/
end
endmodule
