/* This is the simulation for the top file. It simulates the user pressing the reset button on the
DE2 board. Note that we are simulating 8-bit RAM. DDR.v has a compilation error if 16-bit is chosen.
Please contact Micron you want to fix this :)
*/


`include "defs.v"

module test ();

    reg ddr_rst, CLOCK_50;
    
    wire test_done;
    
    initial begin
        $dumpfile("ddr_top.vcd"); //GTKWave outputfile
        $dumpvars(0,test); //dump all signals into .vcd
//        #(`DDR_CLK_PERIOD * 50) $finish;
    end
    initial begin
        CLOCK_50  = 0;
        ddr_rst  = 1;
        #(`DDR_CLK_PERIOD*3);
        ddr_rst = 0;
    end


    always CLOCK_50 = #(`DDR_CLK_PERIOD/2) ~CLOCK_50; 

assign test_done = GPIO_0[0];


always @(test_done)
	if(test_done) 
		#(`DDR_CLK_PERIOD*20) $finish; //finish simulation 20 cycles after read/write is complete



wire	[35:0]	GPIO_0;					//	GPIO Connection 0
wire	[35:0]	GPIO_1;					//	GPIO Connection 1

wire  [12:0] Addr;
assign Addr =
{ 		    GPIO_1[16], //addr 12
					GPIO_1[17], //11
					GPIO_1[7], //10
					GPIO_1[18], //9
					GPIO_1[19], //8
					GPIO_1[20], //7 
					GPIO_1[21], //6
					GPIO_1[22], //5
					GPIO_1[23], //4
					GPIO_1[2], //3
					GPIO_1[4], //2
					GPIO_1[5], //1
					GPIO_1[6]  //addr 0
				}; 


wire [15:0] Dq;
assign  Dq = {	GPIO_0[21], //d15
					GPIO_0[20], //14
					GPIO_0[19], //13
					GPIO_0[18], //12
					GPIO_0[17], //11
					GPIO_0[22], //10
					GPIO_0[23], //9
					GPIO_0[24], //8
					GPIO_0[9], //7
					GPIO_0[10], //6
					GPIO_0[11], //5
					GPIO_0[16], //4
					GPIO_0[15], //3
					GPIO_0[14], //2
					GPIO_0[13], //1
					GPIO_0[12] //0
				}; //bidirectional data io
					

wire [1:0] bank;
assign bank = {GPIO_1[8],GPIO_1[9]} ; //ba1-0

wire CS,RAS,CAS,WE,LDM,LDQS;

assign 		{CS, RAS,CAS,WE,LDM,LDQS} = 
          { GPIO_0[3], //cs
			GPIO_0[4], //ras
			GPIO_0[5], //cas
			GPIO_0[6], //we
			GPIO_0[7], //LDM
			GPIO_0[8] //LDQS
							};


wire UDM, UDQS, CKn, CK, CKE;
assign {CKE, CK, CKn, UDM,UDQS} = GPIO_0[29:25] ; 

//wire CS,RAS,CAS,WE,LDM,LDQS;



DDR_TOP fpga
	(
		////////////////////	Clock Input	 	////////////////////	 
		.CLOCK_27(),						//	27 MHz
		.CLOCK_50(CLOCK_50),						//	50 MHz
		////////////////////	Push Button		////////////////////
		.KEY(),							//	Pushbutton[3:0]
		////////////////////	7-SEG Dispaly	////////////////////
		.HEX0(),							//	Seven Segment Digit 0
		.HEX1(),							//	Seven Segment Digit 1
		.HEX2(),							//	Seven Segment Digit 2
		.HEX3(),							//	Seven Segment Digit 3
		.ddr_rst(ddr_rst),
		.GPIO_0(GPIO_0),						//	GPIO Connection 0
		.GPIO_1(GPIO_1)							//	GPIO Connection 1
	);


	ddr ddram
		(
		.Clk	(CK),
		.Clk_n	(CKn),
		.Cke	(CKE), //not used??
		.Cs_n	(CS), //not used??
 		.Ras_n  (RAS), 
      	.Cas_n  (CAS), 
      	.We_n   (WE), 
`ifdef DDR16BIT // user must choose whether to implement 16 bit or 8 bit ddr
		    // currently, the micron DDR.v module has a compilation error when using 16-bit DDR
		    // For simulation purposes we use 8-bit ram instead 		
      	.Dq     (Dq),
`else
     	.Dq     (Dq[7:0]),
`endif 
        .Addr   (Addr), 
        .Ba     (bank), 	
        .Dm     (LDM), //data mask
        .Dqs    (LDQS) //data strobe
    ); 







endmodule
