//
// file: test_sdram_ctrl.v
//
//    rgb_2 sdram controller (no arbiter) test
//


`include  "global_defs.v"


module test (/*AUTOARG*/ ) ;

    reg sdr_rst, sdr_clk;
    
    initial begin
        $dumpfile("sc.lxt");
        $dumpvars(0, 
                  test.sdramctrl, 
                  test.sdr_clk,
                  test.vi_ff_ren,
                  test.vi_ff_dat_o,
                  test.vo_ff_dat_i,
                  test.sdr_ext_rcw,
                  test.sdr_ext_dat,
                  test.sdr_ext_addr
                  );
//        #(`SDR_CLK_PERIOD * 50) $finish;
    end
    initial begin
        sdr_clk  = 0;
        sdr_rst  = 1;
        #(`SDR_CLK_PERIOD + 3);
        sdr_rst = 0;
    end


    always sdr_clk = #(`SDR_CLK_PERIOD/2) ~sdr_clk; 



    parameter CMD_IDLE = 3'b000,
              CMD_REFR = 3'b001,
              CMD_WR   = 3'b010,
              CMD_RD   = 3'b011,
              CMD_EXT  = 3'b100;

    reg [2:0] sdr_cmd;
    
    reg [10:0] sdr_cmd_addr; // A11 is always set to 0 in sdrctrl
    reg [2:0]  sdr_cmd_rcw;

    reg [7:0]  sdr_len;
    reg [12:0] sdr_row;
    reg [1:0]  sdr_ba;
    reg Cke_cmd;
    reg dqs_cmd;
    wire dqs;
    wire Cke;
    assign dqs = dqs_cmd; 
    assign Cke = Cke_cmd;
    
    initial begin
        vi_ff_dat_o = 8'hzz;
        {sdr_len, sdr_row} = 0;
        dqs = 0;
        sdr_ba = 0;
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = 0;
        Cke_cmd = 0;
        #1;
        wait (!sdr_rst);
        #(`SDR_CLK_PERIOD * 20000); // wait 10 us ! after sdr_clk is functional
        //bring CKE high and execute idle command for 1 cycle
        Cke_cmd = 1;
        sdr_cmd = CMD_IDLE; 
        #(`SDR_CLK_PERIOD * 1);
        // precharge all banks
        $display ("precharge all banks at %t", $time);
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b010, 11'b100_0000_0000};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);
        //set extended mode register
        sdr_ba = 2'b01; //select extended mr
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b000, 11'b000_0000_0000};
         #(`SDR_CLK_PERIOD * 1)
          sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);
        //set mode register
        sdr_ba = 2'b00;
         {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b000, 11'b001_0011_0001};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);
        // precharge all banks
        $display ("precharge all banks at %t", $time);
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b010, 11'b100_0000_0000};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);
        // autorefresh        
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b001, 11'b000_0000_0000};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);
        // autorefresh
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b001, 11'b000_0000_0000};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);  
        // autorefresh
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_EXT,  3'b001, 11'b000_0000_0000};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 9);     
		#(`SDR_CLK_PERIOD * 200);
        wait (test.sdramctrl.done_o);
        
        
        $display("Starting WRITE !!!");
        sdr_len = 3; // reads/2 
        sdr_row = 5;
        dqs_cmd = 1;
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_WR, 3'b000, 11'b000_0000_0000};
        #(`SDR_CLK_PERIOD * 1)
            dqs = 0; 
        	sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
 
        #(`SDR_CLK_PERIOD * 5); 
        wait (test.sdramctrl.done_o);
        $display("Starting READ at time %t", $time);
        sdr_len = 3; // reads/2 
        sdr_row = 5;
        {sdr_cmd, sdr_cmd_rcw, sdr_cmd_addr} = {CMD_RD, 3'b000, 11'b000_0000_0000};
        #(`SDR_CLK_PERIOD * 1) sdr_cmd = CMD_IDLE;// REMOVE CMD !!!
        #(`SDR_CLK_PERIOD * 5); 
        wait (test.sdramctrl.done_o);
        $display("READ done = %d at time %t", test.sdramctrl.done_o, $time);
        
        #(`SDR_CLK_PERIOD * 10);
        $finish;
    end


    // --- vi fifo ----
    wire vi_ff_ren;
    reg  [7:0] vi_ff_dat_o;
    
    always @(posedge sdr_clk)
      if (vi_ff_ren) begin
          if (vi_ff_dat_o === 8'hzz) vi_ff_dat_o = `DLY 0;
          else                      vi_ff_dat_o = `DLY vi_ff_dat_o + 5;
      end
      else begin
          vi_ff_dat_o = `DLY 8'hzz;
      end

    // -- vo fifo ----
    reg [7:0] vo_ff_dat_i;
    
    
    // ------------------

    wire [12:0] sdr_addr;
    wire [2:0] sdr_rcw;
    wire sdr_dts;
    
    sdram_ctrl sdramctrl
      (
       // arbiter interface
       .arb_cmd_i   (sdr_cmd[1:0]),
       .cpu_cmd_i   (sdr_cmd[2]),
      // external command
       .cmd_addr_i (sdr_cmd_addr),  // address for CMD_EXT
       .cmd_rcw_i  (sdr_cmd_rcw),   // ras, cas, we CMD_EXT
       .row_i   (sdr_row),
       .len_i   (sdr_len), // length >> 1
       .done_o  (),
        

       // video I/O
       .ren_o   (vi_ff_ren),
       .wen_o   (),
        
       // sdram interface
       .dts_o  (sdr_dts),  // data buffers tristate
       .addr_o (sdr_addr),
       .rcw_o  (sdr_rcw), // ras, cas, we

       // system   
       .CLK (sdr_clk),
       .RST (sdr_rst)
       
       );

    // -----  IO PADS & REGISTERS ------
    
    reg [2:0] sdr_ext_rcw;

    initial begin
        sdr_ext_rcw = 3'b111;
        {sdr_ext_addr, sdr_ext_ba} = 0;
        sdr_ext_dat_o = 8'hzz;
    end

    always @(posedge sdr_clk)
      sdr_ext_rcw <= `DLY sdr_rcw;


    // pad regs
    reg [7:0] sdr_ext_dat_o;
    always @(posedge sdr_clk)
      sdr_ext_dat_o = `DLY vi_ff_dat_o;

    // tristate pads
    wire [7:0] sdr_ext_dat;
    assign     sdr_ext_dat = sdr_dts == 1 ? 8'hZZ : sdr_ext_dat_o;

    reg [12:0]  sdr_ext_addr;
    reg [1:0]   sdr_ext_ba;
    always @(posedge sdr_clk)
      {sdr_ext_addr, sdr_ext_ba} <= `DLY {sdr_addr, sdr_ba};


    always @(posedge sdr_clk)
      vo_ff_dat_i = `DLY sdr_ext_dat;


    // ----------------------
    
    wire        ras, cas, we, cs, dqm;
    assign {ras, cas, we, cs, dqm} = {sdr_ext_rcw, 1'b0, 1'b0};

/*    
    mt48lc8m8a2 sdram0 
      (
       .Dq    (sdr_ext_dat), 
       .Addr  (sdr_ext_addr), 
       .Ba    (sdr_ext_ba), 
       .Clk   (sdr_clk), 
       .Cke   (1),     // not used 
       .Cs_n  (cs),    // not used 
       .Ras_n (ras), 
       .Cas_n (cas), 
       .We_n  (we), 
       .Dqm   (dqm)      // not used
       );
*/

	wire dqs;

	ddr ddr0
		(
		.Clk	(sdr_clk),
		.Clk_n	(~sdr_clk),
		.Cke	(Cke), //not used??
		.Cs_n	(cs), //not used??
 		.Ras_n  (ras), 
      	.Cas_n  (cas), 
      	.We_n   (we), 
      	.Dq     (sdr_ext_dat), 
        .Addr   (sdr_ext_addr), 
        .Ba     (sdr_ext_ba), 	
        .Dm     (1), //data mask
        .Dqs    (dqs) //data strobe
    ); 

endmodule // test
